"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ALGOImplementation = void 0;
const msgpack = __importStar(require("algo-msgpack-with-bigint"));
const axios_1 = __importDefault(require("axios"));
const __1 = require("..");
const axios_rate_limit_1 = __importDefault(require("../axios-rate-limiter/axios-rate-limit"));
const StatusBase_1 = require("../base-objects/StatusBase");
const TransactionBase_1 = require("../base-objects/TransactionBase");
const types_1 = require("../types");
const algoUtils_1 = require("../utils/algoUtils");
const constants_1 = require("../utils/constants");
const utils_1 = require("../utils/utils");
const sha512_256 = require('js-sha512').sha512_256;
const DEFAULT_TIMEOUT = 60000;
const DEFAULT_RATE_LIMIT_OPTIONS = {
    maxRPS: 5,
};
function algoResponseValidator(responseCode) {
    // allow any response, process them later in mcc
    return responseCode >= 200 && responseCode < 600;
}
class ALGOImplementation {
    constructor(createConfig) {
        this.createConfig = createConfig;
        const algodClient = axios_1.default.create({
            baseURL: createConfig.algod.url,
            timeout: DEFAULT_TIMEOUT,
            headers: {
                "Content-Type": "application/json",
                "X-Algo-API-Token": createConfig.algod.token,
            },
            validateStatus: algoResponseValidator,
        });
        this.algodClient = (0, axios_rate_limit_1.default)(algodClient, Object.assign(Object.assign({}, DEFAULT_RATE_LIMIT_OPTIONS), createConfig.rateLimitOptions));
        if (createConfig.indexer) {
            const indexerClient = axios_1.default.create({
                baseURL: createConfig.indexer.url,
                timeout: DEFAULT_TIMEOUT,
                headers: {
                    "Content-Type": "application/json",
                    "X-Algo-API-Token": createConfig.indexer.token,
                },
                validateStatus: algoResponseValidator,
            });
            this.indexerClient = (0, axios_rate_limit_1.default)(indexerClient, Object.assign(Object.assign({}, DEFAULT_RATE_LIMIT_OPTIONS), createConfig.rateLimitOptions));
        }
        if (!createConfig.inRegTest) {
            this.inRegTest = false;
        }
        else {
            this.inRegTest = createConfig.inRegTest;
        }
        this.loggingObject = createConfig.loggingOptions ? (0, utils_1.fillWithDefault)(createConfig.loggingOptions) : (0, utils_1.defaultMccLoggingObject)();
        this.chainType = types_1.ChainType.ALGO;
    }
    /**
     * @deprecated
     * @returns
     */
    isHealthy() {
        return __awaiter(this, void 0, void 0, function* () {
            let res = yield this.algodClient.get("health"); // TODO all apps must be healthy
            const response_code = res.status;
            return response_code === 200;
        });
    }
    getStatus() {
        return __awaiter(this, void 0, void 0, function* () {
            let res = yield this.algodClient.get("v2/status");
            (0, algoUtils_1.algo_ensure_data)(res);
            return (0, utils_1.toCamelCase)(res.data);
        });
    }
    getBlockHeaderCert(round) {
        return __awaiter(this, void 0, void 0, function* () {
            let res = yield this.algodClient.get(`/v2/blocks/${round}?format=msgpack`, {
                responseType: "arraybuffer",
                headres: { "Content-Type": "application/msgpack" },
            });
            (0, algoUtils_1.algo_ensure_data)(res);
            const decoded = msgpack.decode(res.data);
            return decoded.cert;
        });
    }
    getBlockHeader(round) {
        return __awaiter(this, void 0, void 0, function* () {
            if (round === undefined) {
                const status = yield this.getStatus();
                round = status.lastRound;
            }
            let resorig = yield this.algodClient.get(`/v2/blocks/${round}`);
            // let res = await this.algodClient.get(`/v2/blocks/${round}?format=msgpack`, {responseType: 'arraybuffer', headres: {"Content-Type": "application/msgpack"}});
            // // application/msgpack
            // // responseEncoding: 'ascii',
            // // console.log(res.data);
            // const fs = require('fs')
            // // fs.writeFile('neki.txt', res.data, (err:any) => {
            // //     if (err) {
            // //       console.error(err)
            // //       return
            // //     }
            // //     //file written successfully
            // //   })
            // console.log(res.data)
            // const decoded = msgpack.decode(res.data)
            // // @ts-ignore
            // console.log(decoded);
            // console.log(resorig.data);
            // // hexToBase32()
            // // Block fees (base 32)
            // console.log("fees")
            // // @ts-ignore
            // console.log(hexToBase32(decoded.block.fees))
            // console.log(resorig.data.block.fees);
            // // @ts-ignore
            // console.log(decoded.block.fees)
            // console.log(Buffer.from(hexToBytes(base32ToHex(resorig.data.block.fees))));
            // // Block gh (base 64)
            // console.log("gh")
            // // @ts-ignore
            // console.log(hexToBase64(decoded.block.gh))
            // console.log(resorig.data.block.gh);
            // // @ts-ignore
            // console.log(decoded.block.gh)
            // console.log(Buffer.from(hexToBytes(base64ToHex(resorig.data.block.gh))));
            // // Block prev (base 32)
            // console.log("prev")
            // // @ts-ignore
            // console.log(hexToBase32(decoded.block.prev))
            // console.log(resorig.data.block.prev);
            // // unprefix blk-
            // console.log(resorig.data.block.prev.slice(4));
            // // @ts-ignore
            // console.log(decoded.block.prev)
            // console.log(Buffer.from(hexToBytes(base32ToHex(resorig.data.block.prev))));
            // // unprefix blk-
            // console.log(Buffer.from(hexToBytes(base32ToHex(resorig.data.block.prev.slice(4)))));
            // // Block rwd (base 32)
            // console.log("rwd")
            // // @ts-ignore
            // console.log(hexToBase32(decoded.block.rwd))
            // console.log(resorig.data.block.rwd);
            // // @ts-ignore
            // console.log(decoded.block.rwd)
            // console.log(Buffer.from(hexToBytes(base32ToHex(resorig.data.block.rwd))));
            // console.log(Buffer.from(hexToBytes(base32ToHex(resorig.data.block.rwd))).toString());
            // const hash = crypto.createHash('sha256')
            // // console.log(hash.digest('hex'));
            // hash.update(Buffer.from(hexToBytes(base32ToHex(resorig.data.block.rwd))).toString())
            // console.log(hash.digest('hex'));
            // // Block seed (base 64)
            // console.log("seed")
            // // @ts-ignore
            // console.log(hexToBase64(decoded.block.seed))
            // console.log(resorig.data.block.seed);
            // // @ts-ignore
            // console.log(decoded.block.seed)
            // console.log(Buffer.from(hexToBytes(base64ToHex(resorig.data.block.seed))));
            // // Block txn (base 64)
            // console.log("txn")
            // // @ts-ignore
            // console.log(hexToBase64(decoded.block.txn))
            // console.log(resorig.data.block.txn);
            // // @ts-ignore
            // console.log(decoded.block.txn)
            // console.log(Buffer.from(hexToBytes(base64ToHex(resorig.data.block.txn))));
            // // msgpack specific
            // // @ts-ignore
            // console.log(decoded.cert.prop.dig);
            // // @ts-ignore
            // console.log(hexToBase32(decoded.cert.prop.dig))
            (0, algoUtils_1.algo_ensure_data)(resorig);
            let responseData = (0, utils_1.toCamelCase)(resorig.data);
            responseData.type = "IAlgoGetBlockHeaderRes";
            return responseData;
        });
    }
    getBlock(round) {
        return __awaiter(this, void 0, void 0, function* () {
            if (round === undefined) {
                const status = yield this.getStatus();
                round = status.lastRound;
            }
            let res = yield this.algodClient.get(`/v2/blocks/${round}?format=msgpack`, { responseType: 'arraybuffer', headres: { "Content-Type": "application/msgpack" } });
            if ((0, algoUtils_1.algo_check_expect_block_out_of_range)(res)) {
                return null;
            }
            (0, algoUtils_1.algo_ensure_data)(res);
            const decoded = (0, algoUtils_1.mpDecode)(res.data);
            return new __1.AlgoBlock(decoded);
        });
    }
    getIndexerBlock(round) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.createConfig.indexer) {
                // No indexer 
                return null;
            }
            if (round === undefined) {
                const status = yield this.getStatus();
                round = status.lastRound - 1;
            }
            let res = yield this.indexerClient.get(`/v2/blocks/${round}`);
            if ((0, algoUtils_1.algo_check_expect_block_out_of_range)(res)) {
                return null;
            }
            (0, algoUtils_1.algo_ensure_data)(res);
            const cert = yield this.getBlockHeaderCert(round);
            let camelBlockRes = (0, utils_1.toCamelCase)(res.data);
            camelBlockRes.transactions = [];
            camelBlockRes.type = "IAlgoGetBlockRes";
            for (let key of Object.keys(res.data.transactions)) {
                camelBlockRes.transactions.push((0, utils_1.toCamelCase)(res.data.transactions[key]));
            }
            camelBlockRes.cert = cert;
            return new __1.AlgoIndexerBlock(camelBlockRes);
        });
    }
    // async getBlockHashFromHeight(blockNumber: number): Promise<string | null> {
    //    let header = await this.getBlockHeader(blockNumber);
    //    if (header === null) {
    //       return null;
    //    } else {
    //       console.log(header.cert);
    //    }
    //    // throw Error("Not Implemented")
    //    return null;
    // }
    getBlockHeight() {
        return __awaiter(this, void 0, void 0, function* () {
            const blockData = yield this.getBlockHeader();
            return blockData.block.rnd;
        });
    }
    /**
     * Get trasnaction from node by its id
     * @param txid base32 encoded txid || standardized txid (prefixed or unprefixed)
     * @returns
     */
    getTransaction(txid) {
        return __awaiter(this, void 0, void 0, function* () {
            if (constants_1.PREFIXED_STD_TXID_REGEX.test(txid)) {
                txid = (0, algoUtils_1.hexToBase32)((0, utils_1.unPrefix0x)(txid));
            }
            let res = yield this.indexerClient.get(`/v2/transactions/${txid}`);
            if ((0, algoUtils_1.algo_check_expect_empty)(res)) {
                return null;
            }
            (0, algoUtils_1.algo_ensure_data)(res);
            return new TransactionBase_1.AlgoIndexerTransaction((0, utils_1.toCamelCase)(res.data));
        });
    }
    listTransactions(options) {
        return __awaiter(this, void 0, void 0, function* () {
            let snakeObject = {};
            if (options !== undefined) {
                snakeObject = (0, utils_1.toSnakeCase)(options);
            }
            let res = yield this.indexerClient.get(`/v2/transactions`, {
                params: snakeObject,
            });
            (0, algoUtils_1.algo_ensure_data)(res);
            let camelList = {
                currentRound: res.data["current-round"],
                nextToken: res.data["next-token"],
                transactions: [],
            };
            for (let key of Object.keys(res.data.transactions)) {
                camelList.transactions.push((0, utils_1.toCamelCase)(res.data.transactions[key]));
            }
            return camelList;
        });
    }
    /**
     * TODO implement
     */
    getNodeStatus() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                let res = yield this.algodClient.get("health");
                (0, algoUtils_1.algo_ensure_data)(res);
                let ver = yield this.algodClient.get("versions");
                (0, algoUtils_1.algo_ensure_data)(ver);
                let status = yield this.algodClient.get("/v2/status");
                (0, algoUtils_1.algo_ensure_data)(status);
                status = (0, utils_1.toCamelCase)(status.data);
                let bottomBlockHeight = -1;
                if (res.status === 200) {
                    // Bottom block search
                    bottomBlockHeight = status.lastRound;
                    // check -1.000 -10.000 and -100.000 blocs
                    for (let checkRound = 0; checkRound < 3; checkRound++) {
                        bottomBlockHeight -= Math.pow(10, 3 + checkRound);
                        let blc = yield this.algodClient.get(`/v2/blocks/${bottomBlockHeight}`);
                        if (blc.status !== 200) {
                            // we didn't get block
                            for (let i = 0; i < 10; i++) {
                                bottomBlockHeight += Math.pow(10, 2 + checkRound);
                                blc = yield this.algodClient.get(`/v2/blocks/${bottomBlockHeight}`);
                                if (blc.status === 200) {
                                    break;
                                }
                            }
                            // If we ever come here we are not healthy
                        }
                    }
                }
                const statusData = {
                    health: res.status,
                    bottomBlock: bottomBlockHeight,
                    status: status,
                    versions: (0, utils_1.toCamelCase)(ver.data),
                };
                return new StatusBase_1.AlgoNodeStatus(statusData);
            }
            catch (e) {
                return null;
            }
        });
    }
}
exports.ALGOImplementation = ALGOImplementation;
//# sourceMappingURL=AlgoRpcImplementation.js.map