"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.retry = void 0;
const __1 = require("..");
const utils_1 = require("./utils");
let TIMEOUT_STEP_MULTIPLY = 1.2;
let BACKOFF_TIME_STEP_MULTIPLY = 1.2;
function retry(label, funct, timeoutTime = 5000, numRetries = 5, backOddTimeout = 1000, exceptionCallback = __1.defaultExceptionCallback, warningCallback = __1.defaultWarningCallback, failure = (label) => { }) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            let result = yield Promise.race([funct(), (0, utils_1.sleepMs)(timeoutTime)]);
            if (result)
                return result;
            if (numRetries > 0) {
                warningCallback(`retry ^R${label}^^ ${numRetries}`);
                yield (0, utils_1.sleepMs)(backOddTimeout / 2 + (0, utils_1.getSimpleRandom)(backOddTimeout / 2));
                return retry(label, funct, timeoutTime * TIMEOUT_STEP_MULTIPLY, numRetries - 1, backOddTimeout * BACKOFF_TIME_STEP_MULTIPLY, exceptionCallback, warningCallback, failure);
            }
            else {
                warningCallback(`retry ^R${label}^^ failed`);
                // failure callback
                failure(label);
                //  return null; // TODO this should probably raise an error
                throw (0, __1.MccError)(`retry ^R${label}^^ failed`);
            }
        }
        catch (error) {
            if (numRetries > 0) {
                exceptionCallback(error, `retry ^R${label}^^ exception (retry ${numRetries})`);
                yield (0, utils_1.sleepMs)(backOddTimeout / 2 + (0, utils_1.getSimpleRandom)(backOddTimeout / 2));
                return retry(label, funct, timeoutTime * TIMEOUT_STEP_MULTIPLY, numRetries - 1, backOddTimeout * BACKOFF_TIME_STEP_MULTIPLY, exceptionCallback, warningCallback, failure);
            }
            else {
                exceptionCallback(error, `retry ^R${label}^^ exception (retry failed)`);
                // failure callback
                failure(label);
                //  return null;
                throw (0, __1.MccError)(error);
            }
        }
    });
}
exports.retry = retry;
// export async function retryMany<T>(
//   label: string,
//   functs: ((...args: any) => T)[],
//   timeoutTime: number = 5000,
//   numRetries: number = 5,
//   backOddTimeout: number = 1000,
//   exceptionCallback: IExceptionCallback = defaultExceptionCallback,
//   warningCallback: ILoggingCallback = defaultWarningCallback
// ) {
//    return Promise.all(functs.map((funct) => retry(label, funct, timeoutTime, numRetries, backOddTimeout, exceptionCallback, warningCallback)));
// }
//# sourceMappingURL=retry.js.map